/*
    HTML@Zaurus HTML Editor for Sharp SL5xxx Linux PDA
    Copyright (C) 2002  Scott Taylor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/

#include "include/filedialogs.h"

#include <qlineedit.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>


#include <qdir.h>
#include <qfile.h>
//#include <qfileinfolist.h>
//#include <qfileinfo.h>
//#include <qfileinfolistiterator.h>


/*
 *  Constructs a FileDialog which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
FileDialogS::FileDialogS( QWidget* parent,  const char* name, bool modal, WFlags fl)
{
	FileDialogS(parent, name, modal, fl, TRUE);
}


FileDialogS::FileDialogS( QWidget* parent,  const char* name, bool modal, WFlags fl , bool load)
    : QDialog( parent, name, modal, fl )
{
	LOAD = load;
	if ( !name )
	{
		setName( "FileDialog" );
	}
	resize( 204, 195 );
	setCaption( tr( "Choose File..." ) );

	ListBox1 = new QListBox( this, "ListBox1" );
	//ListBox1->insertItem( tr( "New Item" ) );
	ListBox1->setGeometry( QRect( 5, 10, 193, 120 ) );

	LineEdit1 = new QLineEdit( this, "LineEdit1" );
	LineEdit1->setGeometry( QRect( 5, 140, 193, 22 ) );

	PushButton1 = new QPushButton( this, "PushButton1" );
	PushButton1->setGeometry( QRect( 10, 170, 81, 21 ) );

	if(load)
	{
		PushButton1->setText( tr( "Open" ) );
	}
	else
	{
		PushButton1->setText( tr( "Save" ) );
	}

	PushButton1_2 = new QPushButton( this, "PushButton1_2" );
	PushButton1_2->setGeometry( QRect( 110, 170, 81, 21 ) );
	PushButton1_2->setText( tr( "Cancel" ) );

        d.setFilter(  QDir::All );	//QDir::Files |
        d.setSorting( QDir::DirsFirst);

	connect(ListBox1 ,SIGNAL(clicked( QListBoxItem * )), this, SLOT(changeList( QListBoxItem * ))  );
	connect(PushButton1 ,SIGNAL(clicked()), this, SLOT(fileSelected())  );
	connect(PushButton1_2 ,SIGNAL(clicked()), this, SLOT(hide())  );

	setList("~/");

}


void FileDialogS::setLoad(bool yesNo)
{
	LOAD = yesNo;
	if(yesNo)
	{
		PushButton1->setText( tr( "Open" ) );
	}
	else
	{
		PushButton1->setText( tr( "Save" ) );
	}
	
}


void FileDialogS::fileSelected()
{
	if(LOAD)
	{		
		emit doneLoad();
	}
	else
	{
		emit done();
	}
	hide();
}


void FileDialogS::changeList(QListBoxItem *myItem)
{
	//printf("Hello");
	QString myDir = ListBox1->currentText();
	//printf("Dir = %s\n",myDir.latin1());
	if(myDir == "..")
	{
		d.cdUp();
		QString myPath = d.absPath();
		currentDir = myPath.latin1();
		returnFileName = currentDir;
		//printf("MyPath :%s\n",currentDir);
		setList(d.absPath().latin1());
	}
	else if(myDir.latin1() == ".")
	{

	}
	else
	{

		QString myPath = d.absPath();
		myPath.append("/");
		myPath.append(myDir.latin1());
		currentDir = myPath.latin1();
		returnFileName = currentDir;
		if(!d.cd(currentDir))
		{
			currentFileName = myDir.latin1();
			LineEdit1->setText(myDir.latin1());
		}
		else
		{

			setList(myPath.latin1());
			//printf("myPath:%s\n",d.absPath().latin1());
		}
	}
}


void FileDialogS::setList(const char *myDir)
{

	//printf("1\n");
	ListBox1->clear();
	//printf("2\n");
 	const QFileInfoList *list = d.entryInfoList();
	//printf("3\n");
        QFileInfoListIterator it( *list );      // create list iterator
	//printf("4\n");
        QFileInfo *fi;                          // pointer for traversing
	//printf("5\n");
        while ( (fi=it.current()) )
	{           // for each file...
		ListBox1->insertItem( fi->fileName().data() );
            	++it;                               // goto next list element
        }
	//printf("6\n");
}


QString * FileDialogS::getSelectedFileName()
{
	return new QString(LineEdit1->text());
}
QString * FileDialogS::getAbsSelectedFileName()
{
	QString *myDir = new QString(LineEdit1->text());
	QString *myPath = new QString(d.absPath());
	myPath->append("/");
	myPath->append(myDir->latin1());
	
	delete myDir;
	//printf("Retuning %s\n",myPath.latin1());
	return myPath;
}





/*
 *  Destroys the object and frees any allocated resources
 */
FileDialogS::~FileDialogS()
{
    // no need to delete child widgets, Qt does it all for us
}

