/*
    HTML@Zaurus HTML Editor for Sharp SL5xxx Linux PDA
    Copyright (C) 2002  Scott Taylor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/

#include "include/htmlconfig.h"

#include <qlineedit.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qpe/global.h>

#include <qdir.h>
#include <qfile.h>

#include <stdlib.h>

/***********************************************************************
   Constructs a HTMLConfig which is a child of 'parent', with the
   name 'name' and widget flags set to 'f'
 
   The dialog will by default be modeless, unless you set 'modal' to
   TRUE to construct a modal dialog.

***********************************************************************/

HTMLConfig::HTMLConfig( QWidget* parent,  const char* name, bool modal, WFlags fl)
    : QDialog( parent, name, modal, fl )
{
	
	if ( !name )
	{
		setName( "Config" );
	}
	resize( 204, 160 );
	setCaption( tr( "HTML@Zaurus Config" ) );

	QLabel *qlTmp = new QLabel(this);
	qlTmp->setGeometry( QRect( 5, 1, 173,15) );
	qlTmp->setText( tr("Browser Path...") );


	
	browserName = new QLineEdit( this, "browserName" );
	browserName->setGeometry( QRect( 5, 15, 193, 20 ) );

	upperCase = new QCheckBox( this, "UpperCase" );
	upperCase->setGeometry( QRect( 5, 45, 193, 20 ) );
	upperCase->setText( tr( "Upper Case Tags?" ) );

	lineWrap = new QCheckBox( this, "LineWrap" );
	lineWrap->setGeometry( QRect( 5, 75, 193, 20 ) );
	lineWrap->setText( tr( "Wrap lines?" ) );


	wrapAt = new QLineEdit( this, "WrapAt");
	wrapAt->setGeometry( QRect(5,105,20,20));
	QLabel *qlTmp2 = new QLabel(this);
	qlTmp2->setGeometry( QRect( 35, 105, 173,20) );
	qlTmp2->setText( tr("Wrap line at?") );
	
	
	saveButton = new QPushButton(this, "SaveButton");
	saveButton->setGeometry( QRect( 5, 135, 193, 20 ) );
	saveButton->setText( tr( "Save" ) );

	loadConfig();
	connect(saveButton ,SIGNAL(clicked()), this, SLOT(saveConfig())  );
}

void HTMLConfig::loadConfig()
{
	//QFile configFile(Global::applicationFileName(APP_NAME, CONF_FILE).latin1());
	QFile configFile(CONF_FILE);
	
	QString *conf = new QString();
	
	UPPERCASE = false;
	if(configFile.open(IO_ReadOnly))
	{ 

		char *nextConf = (char *)malloc(512);
		while( configFile.readLine(nextConf,512) != -1 )
		{
			conf->append(nextConf);			
			if(conf->startsWith("#browser:"))
			{
				QString browserS = conf->mid(9,(conf->length()-1));				
				browserName->setText((browserS.stripWhiteSpace()).latin1());
			}
			else if(conf->startsWith("#upperCase:true"))
			{
				UPPERCASE = true;
				upperCase->setChecked(TRUE);
			}
			else if(conf->startsWith("#lineWrap:"))
			{
				lineWrap->setChecked(TRUE);
				QString wrap = conf->mid(10,(conf->length()-1));				
				wrapAt->setText((wrap.stripWhiteSpace()).latin1());	
			}
			

			conf = new QString();
			free(nextConf);
			nextConf = (char *)malloc(512);	//Do I need to free it and then reasign it?
		}
		configFile.close();
	}
	else
	{
		printf("File not found %s",Global::applicationFileName(APP_NAME, CONF_FILE).latin1());
	}
}




void HTMLConfig::saveConfig()
{
        QString *text = new QString();

		text->append("#browser:");
		text->append(browserName->text().latin1());
		text->append("\n");
		text->append("#upperCase:");
		if(upperCase->isChecked())
		{
			text->append("true");
		}
		else
		{
			text->append("false");
		}
		text->append("\n");
		if(lineWrap->isChecked())
		{
			text->append("#lineWrap:");
			text->append(wrapAt->text().latin1());
			text->append("\n");
		}

        
		//QFile configFile(Global::applicationFileName(APP_NAME, CONF_FILE).latin1());
		QFile configFile(CONF_FILE);

        if ( !configFile.open( IO_WriteOnly ) )
		{
	    	QMessageBox::warning( this, "Error",
                                "You do not have write permission\nhere.\n\n",
                          	"Ok", "Quit", 0,
                          	0, 1 );
            return;
        }
        QTextStream t( &configFile );
        t << text->latin1();
        configFile.close();
        if(upperCase->isChecked() != UPPERCASE)
        {
			QMessageBox::warning( this, "Info",
                            "Changes to Tag case will be active\nat the next start!\n",
                      		"Ok", "Quit", 0,
                      		0, 1 );
        }
        
        hide();
}




/***********************************

Destroys the object and frees any 
allocated resources

************************************/
HTMLConfig::~HTMLConfig()
{
    // no need to delete child widgets, Qt does it all for us
}

