/*
    HTML@Zaurus HTML Editor for Sharp SL5xxx Linux PDA
    Copyright (C) 2002  Scott Taylor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/


#include "include/htmlEditor.h"
#include "include/colorDialog.h"


#include <stdlib.h>	
#include <unistd.h>
#include <stdio.h>
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qtextstream.h>
#include <qpixmap.h>
#include <qpe/fileselector.h>
#include <qpe/global.h>
#include <qpe/resource.h>


/*
 *  Constructs a HTMLEditor which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'fl'
 */
HTMLEditor::HTMLEditor( QWidget* parent,  const char* name, WFlags fl )
	: QMainWindow( 0, "example application main window", WDestructiveClose )
{
	UPPER = FALSE;
	wrapAt = 0;
	loadConfig();
	CLOSE = false;
	listBox = new QComboBox(this,"myList");//printf("1\n");
	
	dialog = new HTMLPropertyDialog(this, "dialog", TRUE, WType_Modal);		
	tagDialog = new HTMLPTagDialog(this, "tags",TRUE, WType_Modal, TRUE);	
	f = new FileDialogS(this, "save",TRUE, WType_Modal, TRUE);	

	fileName = new QString();
	tagCount = 0;
	tag = new HTMLTag[MAX_TAGS];

	tagInit();
	int toChoose = 0;

    //Now popultate the tag listbox with the tag names
    //in the array
	for(int i = 0; i < tagCount; i++,toChoose++)
	{
		listBox->insertItem(tag[i].getKey()->latin1(),toChoose);
	}

	//Do layout stuff
	if ( !name )
	{
		setName( "HTML@Zaurus" );
	}

	resize( 196, 245 );
	setCaption( tr( "HTML@Zaurus" ) );

	QMenuBar *theBar = menuBar();
 	QPopupMenu *file = new QPopupMenu( this );
 	QPopupMenu *edit = new QPopupMenu( this );
	QPopupMenu *insert = new QPopupMenu(this);
	QPopupMenu *help = new QPopupMenu(this);

	theBar->insertItem( "&File", file );
	theBar->insertItem( "&Edit", edit );
	theBar->insertItem( "&Insert", insert );
	theBar->insertItem( "&Help", help );

	//Now using Resource for the pixmaps...Test
	const QPixmap *openD = new QPixmap( Resource::loadPixmap( "htmlatzaurus/icons/open.png" ));
	file->insertItem(*openD, "&Open", this, SLOT(showFileChooserLoad()));//, CTRL+Key_O );
	delete openD;
	const QPixmap *newD = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/new.png"));
	file->insertItem(*newD, "&New", this, SLOT(newDocument()));//, CTRL+Key_O );
	delete newD;

	const QPixmap *preview = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/preview.png"));
	file->insertItem(*preview, "&Preview", this, SLOT(preview()));
	delete preview;
	file->insertSeparator();

	const QPixmap *saveD = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/save.png"));
	const QPixmap *saveD2 = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/save.png"));
	file->insertItem(*saveD,"&Save", this, SLOT(save()));//, CTRL+Key_Q);
	delete saveD;
	file->insertItem(*saveD2,"Save &As", this, SLOT(showFileChooserSave()));//, CTRL+Key_Q);
	delete saveD2;
	file->insertSeparator();
	
	const QPixmap *config = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/configure.png"));
	file->insertItem(*config,"&Configuration", this, SLOT(showConfig()));//, CTRL+Key_Q);
	delete config;
	
	file->insertSeparator();
	const QPixmap *quitD = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/quit.png"));
	file->insertItem(*quitD,"&Quit", this, SLOT(goodBye()));//, CTRL+Key_Q);
	delete quitD;

	const QPixmap *undo = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/undo.png"));
	edit->insertItem(*undo, "&Undo", this, SLOT(undo()));
	delete undo;
	
	const QPixmap *redo = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/redo.png"));
	edit->insertItem(*redo,"&Redo", this, SLOT(redo()));
	delete redo;
	
	edit->insertSeparator();
	
	const QPixmap *cut = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/cut.png"));
	edit->insertItem(*cut, "&Cut", this, SLOT(cut()));
	delete cut;
	
	const QPixmap *copy = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/copy.png"));
	edit->insertItem(*copy, "&Copy", this, SLOT(copy()));
	delete copy;
	
	const QPixmap *paste = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/paste.png"));
	edit->insertItem(*paste, "&Paste", this, SLOT(paste()));
	delete paste;



	const QPixmap *blankD = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/new.png"));
	insert->insertItem(*blankD,"&Blank Page", this, SLOT(blankDocument()));	
	delete blankD;
	
	const QPixmap *colorD = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/color.png"));
	insert->insertItem(*colorD,"&Color",this,SLOT(colorDial()));
	delete colorD;
	
	const QPixmap *personal = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/personal.png"));
	insert->insertItem(*personal ,"&Personalized Tag", this, SLOT(insertPTag()));
	delete personal;

	const QPixmap *helpp = new QPixmap(Resource::loadPixmap( "htmlatzaurus/icons/help.png"));
	help->insertItem(*helpp, "&About", this, SLOT(about()));	//tagCount = handler->getTagCount();
	delete helpp;

	inserter = new QPushButton(this, "insertTag" );
	inserter->setText( tr( "Insert Tag" ) );
	editor = new HTMLMLineEdit(this,"Editor");

	inserter->setGeometry( QRect( 160, 30, 71, 20 ) );
	listBox->setGeometry( QRect( 5, 30, 151, 20 ) );
	editor->setGeometry( QRect( 5, 55, 227, 230 ) );
	
	if(wrapAt != 0)
	{
		editor->setWordWrap(editor->FixedColumnWidth);
		editor->setWrapColumnOrWidth(wrapAt);	
	}
	//tagCount = handler->getTagCount();

	//Set the Slots
	connect(inserter, SIGNAL(clicked()),this, SLOT(openPropertiesDialog()));
	connect(dialog, SIGNAL(done()),this, SLOT(insertTag()));
	connect(f ,SIGNAL(doneLoad()), this, SLOT(load())  );
	connect(f ,SIGNAL(done()), this, SLOT(saveAs())  );
	
	connect(tagDialog ,SIGNAL(done()), this, SLOT(insertMyTag())  );
	
	//docDone();  //What was I thinking....???
}


void HTMLEditor::tagInit()
{

	//Here we create all the tags....
	handler = new HTML_XMLParser(UPPER);

    //Give the xml parser a empty array to fill with created tags.
	handler->setTagArray(tag);
    //Get the xml file from global
    QFile *xmlFile = new QFile(TAG_XML_FILE);
    //QFile *xmlFile = new QFile(Global::applicationFileName(APP_NAME, TAG_XML_FILE).latin1());
    //QFile xmlFile( Global::applicationFileName(APP_NAME, TAG_XML_FILE) );
    QXmlInputSource source( *xmlFile );
    QXmlSimpleReader *reader = new QXmlSimpleReader();
    reader->setContentHandler( handler );
    reader->parse( source );
	while(!handler->docDone())
	{
		printf("waiting...\n");// Wait for the parser to finish
	}
	tagCount = handler->getTagCount();
	//printf("\nAmount of Tags: %d", tagCount);

	tagDialog->setTags(tag,tagCount);
	//delete &reader;
	//delete &source;	
	//delete xmlFile;
	xmlFile->close();	
	delete reader;
	//delete handler;
}

void HTMLEditor::copy()
{
	editor->copy();
}

void HTMLEditor::paste()
{
	editor->paste();
}

void HTMLEditor::cut()
{
	editor->cut();
}

void HTMLEditor::undo()
{
	editor->undo();
}

void HTMLEditor::redo()
{
	editor->redo();
}


void HTMLEditor::showConfig()
{
    HTMLConfig	*confDial = new HTMLConfig(this, "tags",TRUE, WType_Modal);
	confDial->exec();//show();
	loadConfig();
	if(wrapAt == 0)
	{
		editor->setWordWrap(editor->NoWrap);
	}
	else
	{
		editor->setWordWrap(editor->FixedColumnWidth);
		editor->setWrapColumnOrWidth(wrapAt);
	}
	delete confDial;
	//tagInit();
	
}


void HTMLEditor::insertPTag()
{

	//tagDialog = new HTMLPTagDialog(this, "tags",TRUE, WType_Modal, TRUE);	
	tagDialog->exec();//show();
	
}

void HTMLEditor::insertMyTag()
{
	editor->insert(tagDialog->getSelectedTag());
}

void HTMLEditor::loadConfig()
{
	QFile *configFile = new QFile(CONF_FILE);
//	QFile *configFile = new QFile(Global::applicationFileName(APP_NAME, CONF_FILE).latin1());
	//QFile configFile (Global::applicationFileName(APP_NAME, CONF_FILE).latin1());
		
	if(configFile->open(IO_ReadOnly))
	{ 
		char *nextConf = (char *)malloc(512);
		while( configFile->readLine(nextConf,512) != -1 )
		{
			QString *conf = new QString();			
			conf->append(nextConf);		
			if(conf->startsWith("#browser:"))
			{
				QString *browserS = new QString();
				browserS->append(conf->mid(9,(conf->length()-1)));
				
				browser = new QString((browserS->stripWhiteSpace()).latin1());
				delete browserS;
			}
			else if(conf->startsWith("#upperCase:true"))
			{
				UPPER = TRUE;
			}
			else if(conf->startsWith("#lineWrap:"))
			{
				QString *wrap = new QString();
				
				wrap->append(conf->mid(10,(conf->length()-1)));				
				wrap->stripWhiteSpace().latin1();	
				bool ok = true;			
				wrapAt = wrap->toInt(&ok,10);
				delete wrap;
			}
			delete conf;
			free(nextConf);
			nextConf = (char *)malloc(512);	//Do I need to free it and then reasign it?
		}
		free(nextConf);
		configFile->close();
	}
	else
	{
		printf("File not found %s",Global::applicationFileName(APP_NAME, CONF_FILE).latin1());
	}
}


void HTMLEditor::preview()
{
	if(browser)
	{
		save(TEMP_FILE);		
		char *mychars;
		mychars = (char *)malloc(512);
		if(!mychars)
		{
			printf("Memory exhausted\n");
			exit(1);
		}
		sprintf(mychars,"%s %s &", browser->latin1(), TEMP_FILE);
		callSystem(mychars);
		free(mychars);
	}
	else
	{
		QMessageBox::warning( this, "Error",
                            "You must first specify a browser.\n\n",
                      		"Ok", "Quit", 0,
                      		0, 1 );
	}
}
void HTMLEditor::callSystem(char *string)
{
	system(string);
}

void HTMLEditor::docDone()
{
}



void HTMLEditor::colorDial()
{
	QColor *color;
	color = new QColor(ColorDialog::getColor(QColor(255, 255, 255),this, "colorD"));

	QString *colorS = new QString();
	char *myRED = (char *)malloc(2*sizeof(char));
	char *myGREEN = (char *)malloc(2*sizeof(char));
	char *myBLUE = (char *)malloc(2*sizeof(char));
	sprintf(myRED,"%02x",color->red());
	sprintf(myGREEN,"%02x",color->green());
	sprintf(myBLUE,"%02x",color->blue());
	colorS->append("#");
	colorS->append(myRED);
	colorS->append(myGREEN);
	colorS->append(myBLUE);
	editor->insert(colorS->latin1());

	free(myRED);
	free(myGREEN);
	free(myBLUE);
	delete colorS;	
	delete color;
}

void HTMLEditor::closeEvent ( QCloseEvent * e )
{
	if(!editor->edited())
	{
		e->accept();
	}
	else
	{
		int ret = QMessageBox::warning( this, "Unsaved Data",
                                "Would you like to save your \nchanges.\n\n",
                          	"Yes", "No","Cancel", 0,2 );

		switch(ret)
		{
			case 0:
				save();
				e->accept();
				break;
			case 1:
				e->accept();
				break;
			case 2:
				break;
		}
	}
}

/*
 *  The goodBye Slot
 */
void HTMLEditor::goodBye()
{
	close();
}


void HTMLEditor::newTag(const char *tagName)
{
	HTMLTag *tempTag = new HTMLTag(tagName, UPPER);
	tag[tagCount] = *tempTag;
	tagCount++;
}

void HTMLEditor::openPropertiesDialog()
{
	dialog->makeVisible(TRUE);
	dialog->setTag(&tag[listBox->currentItem()]);
	if(editor->hasMarkedTxt())
	{
		dialog->setMarkedText(editor->markedTxt());
	}
}


void HTMLEditor::insertTag()	//
{
	HTMLTag *chosenTag = dialog->getFinishedTag();//&tag[tmp];
	editor->insert(chosenTag->getText()->latin1());
	chosenTag->clearProperties();
//??	free(chosenTag);
}

void HTMLEditor::showFileChooserLoad()
{
	f->setLoad(TRUE);
	//connect(f ,SIGNAL(done()), this, SLOT(load())  );
	f->exec();
}

void HTMLEditor::showFileChooserSave()
{

	f->setLoad(FALSE);
	//connect(f ,SIGNAL(done()), this, SLOT(saveAs())  );
	f->exec();	
}


void HTMLEditor::load()
{
	if ( !editor->edited() )
	{
		QString *file = f->getAbsSelectedFileName();

		load(file->latin1());
		delete file;
	}
	else
	{
		int ret = QMessageBox::warning( this, "Unsaved Data",
                                "Would you like to save your \nchanges.\n\n",
                          	"Yes", "No","Cancel", 0,2 );

		switch(ret)
		{
			case 0:
				save();
				load();
				break;
			case 1:
				editor->setEdited( FALSE );
				load();
				break;
			case 2:
				break;
		}
	}

}


void HTMLEditor::load( const char *fileN )
{
	fileName = new QString(fileN);
	QFile fil( fileN );
	//free(fileN);
	
	if ( !fil.open( IO_ReadOnly ) )
	{
		QMessageBox::warning( this, "Error",
                                "You do not have read permission\nhere.\n\n",
                          	"Ok", "Quit", 0,
                          	0, 1 );

		return;
	}
	editor->setAutoUpdate( FALSE );
	editor->clear();

	QTextStream t(&fil);
	while ( !t.eof() )
	{
		QString *s = new QString(t.readLine());
		editor->append( s->latin1() );

		delete s;
	}
	fil.close();
	editor->setAutoUpdate( TRUE );
	editor->repaint();
	editor->setEdited( FALSE );
}

void HTMLEditor::saveAs()
{
	fileName = new QString(f->getAbsSelectedFileName()->latin1());
	save();
}

void HTMLEditor::save()
{
    if ( fileName->isEmpty() )
	{
        showFileChooserSave();
        return;
    }

    QString *text = new QString(editor->text().latin1());
    QFile f( *fileName );
    if ( !f.open( IO_WriteOnly ) )
	{
    	QMessageBox::warning( this, "Error",
                            "You do not have write permission\nhere.\n\n",
                      		"Ok", "Quit", 0,
                      		0, 1 );
        return;
    }
    QTextStream t( &f );
    t << text->latin1();

    delete text;
    f.close();
	editor->setEdited( FALSE );
}
void HTMLEditor::save(const char *fileN)
{
    QString *text = new QString(editor->text().latin1());
    QFile f( fileN );
    if ( !f.open( IO_WriteOnly ) )
	{
    	QMessageBox::warning( this, "Error",
                            "You do not have write permission\nhere.\n\n",
                      		"Ok", "Quit", 0,
                      		0, 1 );
        return;
    }
    QTextStream t( &f );
    t << text->latin1();
    delete *text;
    f.close();
}



void HTMLEditor::blankDocument()
{
	const char *htmlPage = "<html>\n   <head>\n      <title></title>\n   </head>\n   <body>\n   </body>\n</html>";
    if ( !editor->edited() )
	{
		editor->clear();
		editor->insert(htmlPage);
    }
	else
	{
		int ret = QMessageBox::warning( this, "Unsaved Data",
                                "Would you like to save your \nchanges.\n\n",
                          	"Yes", "No","Cancel", 0,2 );

		switch(ret)
		{
			case 0:
				save();
				editor->clear();
				editor->insert(htmlPage);
				//editor->setEdited( FALSE );
				break;
			case 1:
				editor->clear();
				editor->insert(htmlPage);
				//editor->setEdited( FALSE );
				break;
			case 2:
				break;
		}
	}
}



void HTMLEditor::newDocument()
{
    if ( !editor->edited() )
	{
		editor->clear();
		editor->setEdited( FALSE );
    }
	else
	{
		int ret = QMessageBox::warning( this, "Unsaved Data",
                                "Would you like to save your \nchanges.\n\n",
                          	"Yes", "No","Cancel", 0,2 );

		//printf("%d\n",ret);
		switch(ret)
		{
			case 0:
				save();
				editor->clear();
				editor->setEdited( FALSE );
				break;
			case 1:
				editor->clear();
				editor->setEdited( FALSE );
				break;
			case 2:
				break;
		}
	}
}

void HTMLEditor::about()
{
	QString *about = new QString();
	//QFile aboutFile(Global::applicationFileName("htmlatzaurus", "about.txt").latin1());
	QFile aboutFile(ABOUT_FILE);
	if(aboutFile.open(IO_ReadOnly))
	{
		QTextStream t(&aboutFile);
		while ( !t.eof() )
		{
			QString *s = new QString(t.readLine().latin1());
			about->append( s->latin1() );
			about->append("\n");
			delete s;
		}
		aboutFile.close();
	}		
	QMessageBox::information(this, "About",	tr( about->latin1() )	);
	delete about;
}



/*
 *  Destroys the object and frees any allocated resources
 */
HTMLEditor::~HTMLEditor()
{
	delete f;
	delete dialog;
	delete tagDialog;
    free(tag); //The tags consume the most amount of memory...
    // no need to delete child widgets, Qt does it all for us
}



