/*
    HTML@Zaurus HTML Editor for Sharp SL5xxx Linux PDA
    Copyright (C) 2002  Scott Taylor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/

#include "include/htmlptagdialog.h"
#include "include/htmltag.h"

#include <qlineedit.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qpe/global.h>

#include <qdir.h>
#include <qfile.h>

#include <stdlib.h>

/***********************************************************************
   Constructs a HTMLPTagDialog which is a child of 'parent', with the
   name 'name' and widget flags set to 'f'
 
   The dialog will by default be modeless, unless you set 'modal' to
   TRUE to construct a modal dialog.

***********************************************************************/

HTMLPTagDialog::HTMLPTagDialog( QWidget* parent,  const char* name, bool modal, WFlags fl)
{
	HTMLPTagDialog(parent, name, modal, fl, TRUE);
}


HTMLPTagDialog::HTMLPTagDialog( QWidget* parent,  const char* name, bool modal, WFlags fl , bool load)
    : QDialog( parent, name, modal, fl )
{

	dialog = new HTMLPropertyDialog(this, "dialog", TRUE, WType_Modal);
	
	if ( !name )
	{
		setName( "Personalized Tags" );
	}
	resize( 204, 215 );
	setCaption( tr( "Choose Tag..." ) );

	ListBox1 = new QListBox( this, "ListBox1" );
	ListBox1->setGeometry( QRect( 5, 10, 170, 120 ) );

	DeleteButton = new QPushButton(this, "DeleteButton");
	DeleteButton->setGeometry( QRect( 180, 10, 20, 120 ) );
	DeleteButton->setText( tr( "D\ne\nl\ne\nt\ne" ) );

	
	listBox = new QComboBox(this,"myList");
	listBox->setGeometry( QRect( 10, 135, 100, 20 ) );
	
	NewButton = new QPushButton(this, "NewButton");
	NewButton->setGeometry( QRect( 110, 133, 80, 24 ) );
	NewButton->setText( tr( "New Tag" ) );
	



	LineEdit1 = new QLineEdit( this, "LineEdit1" );
	LineEdit1->setGeometry( QRect( 5, 160, 193, 20 ) );

	PushButton1 = new QPushButton( this, "PushButton1" );
	PushButton1->setGeometry( QRect( 10, 185, 80, 20 ) );

	PushButton1->setText( tr( "Insert" ) );

	PushButton1_2 = new QPushButton( this, "PushButton1_2" );
	PushButton1_2->setGeometry( QRect( 110, 185, 80, 21 ) );
	PushButton1_2->setText( tr( "Cancel" ) );



	connect(ListBox1 ,SIGNAL(clicked( QListBoxItem * )), this, SLOT(changeList( QListBoxItem * ))  );
	connect(PushButton1 ,SIGNAL(clicked()), this, SLOT(tagSelected())  );
	connect(PushButton1_2 ,SIGNAL(clicked()), this, SLOT(hide())  );
	
	connect(NewButton ,SIGNAL(clicked()), this, SLOT(newTag())  );
	connect(DeleteButton ,SIGNAL(clicked()), this, SLOT(deleteTag())  );
	connect(dialog, SIGNAL(done()),this, SLOT(insertTag()));

	setList();

}

void HTMLPTagDialog::setTags(HTMLTag *tags, int count)
{
	mytags = tags;
	for(int i = 0; i < count; i++)
	{
		listBox->insertItem((mytags+i)->getKey()->latin1(),i);
	}
}
void HTMLPTagDialog::deleteTag()
{
		int ret = QMessageBox::warning( this, "Warning!",
                                "Do you realy want to delete?.\n\n",
                          	"Yes", "No","Cancel", 0,2 );

		switch(ret)
		{
			case 0:
				ListBox1->removeItem(ListBox1->currentItem());
				//save(Global::applicationFileName("htmlatzaurus", "myTags.dat").latin1());
				save(PERSONALIZED_TAGS_FILE);
				break;
			case 1:
				break;
			case 2:
				break;
		}
}

void HTMLPTagDialog::newTag()
{
	dialog->makeVisible(TRUE);
	int tmp = listBox->currentItem();
	HTMLTag *chosenTag = (mytags+tmp);
	dialog->setTag(chosenTag);
}


void HTMLPTagDialog::insertTag()	//
{
	HTMLTag *chosenTag = dialog->getFinishedTag();
	if(chosenTag)
	{
		ListBox1->insertItem(chosenTag->getText()->latin1());
		chosenTag->clearProperties();
		//save(Global::applicationFileName("htmlatzaurus", "myTags.dat").latin1());
		save(PERSONALIZED_TAGS_FILE);
	}
}

void HTMLPTagDialog::tagSelected()
{
	emit done();
	hide();
}

/***********************************

SLOT:Inserts the selected tag's text
into the LineEdit.

************************************/

void HTMLPTagDialog::changeList(QListBoxItem *myItem)
{
	if(myItem)
	{
		LineEdit1->setText(myItem->text());
	}
}



/***********************************

Retrieves the personalized tags from 
the myTags file and presents all the 
presaved tags for selection

************************************/
void HTMLPTagDialog::setList()
{
	ListBox1->clear();
	//QFile tagFile(Global::applicationFileName(APP_NAME, PERSONALIZED_TAGS_FILE).latin1());
	QFile tagFile(PERSONALIZED_TAGS_FILE);
	
	QString *tagString = new QString();
	QString *tagStringIns = new QString();
	
	bool TAG_OPEN = FALSE;
	
	if(tagFile.open(IO_ReadOnly))
	{
		char *nextTag = (char *)malloc(512);
		int at = 0;
		while( tagFile.readLine(nextTag,512) != -1 )
		{
			tagString->append(nextTag);
			
			//printf("TagString %s\n",tagString->latin1());
			//printf("Tag is Open %d\n",TAG_OPEN);			
			if(tagString->startsWith("#tagStart:"))
			{
				//printf("\nContains #tagStart:");
				TAG_OPEN = TRUE;
				tagString = new QString();
			}
			else if(tagString->startsWith(":tagEnd#"))
			{
				//printf("\nContains #tagEnd:");
				TAG_OPEN = FALSE;
				tagString = new QString();
				//printf("\nInserting-->%s",tagStringIns->latin1());
				ListBox1->insertItem(tagStringIns->latin1());				
				tagStringIns= new QString();
			}
			else if(TAG_OPEN)
			{
				//printf("\nTag_OPEN");
				tagStringIns->append(tagString->latin1());
				tagString = new QString();
			}			

			
			free(nextTag);
			nextTag = (char *)malloc(512);	//Do I need to free it and then reasign it?
		}
		tagFile.close();
	}
}

/***********************************

Returns the selected tags text.

************************************/

const char * HTMLPTagDialog::getSelectedTag()
{
	return LineEdit1->text().latin1();   
}



void HTMLPTagDialog::save(const char *fileName)
{
        QString *text = new QString();
        int count = ListBox1->count();
        
        for(int i = 0; i < count; i++)
        {        	
			ListBox1->setCurrentItem ( i );
			text->append("#tagStart:\n");
			text->append(ListBox1->currentText());
			text->append("\n:tagEnd#\n");

        }
        
        
        QFile f( fileName );
        if ( !f.open( IO_WriteOnly ) )
		{
            //functionNotFin();
	    	QMessageBox::warning( this, "Error",
                                "You do not have write permission\nhere.\n\n",
                          	"Ok", "Quit", 0,
                          	0, 1 );
            return;
        }
        QTextStream t( &f );
        t << text->latin1();
        f.close();
}




/***********************************

Destroys the object and frees any 
allocated resources

************************************/
HTMLPTagDialog::~HTMLPTagDialog()
{
    // no need to delete child widgets, Qt does it all for us
}

