/*
    HTML@Zaurus HTML Editor for Sharp SL5xxx Linux PDA
    Copyright (C) 2002  Scott Taylor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/


#include "include/htmlpropertydialog.h"
#include "include/colorDialog.h"
#include <qcolor.h>
#include <stdlib.h>	
#include <unistd.h>
#include <stdio.h>

#include <qmessagebox.h>



HTMLPropertyDialog::HTMLPropertyDialog( QWidget * parent, const char * name, bool modal, WFlags f )
	: QDialog( parent, name, modal, f )
{

	setFixedSize(200,270);

	propertyKeysLB 		= new QComboBox(this, "keys");
	possibleValuesLB 	= new QComboBox(this, "values");
	lineEd			= new QLineEdit(this, "edit");
	close 			= new QPushButton(this, "Done" );
	add 			= new QPushButton(this, "Add" );
	//preview			= new QPushButton(this, "Preview");
	text			= new QMultiLineEdit(this,"multi");
	addColor		= new QPushButton(this, "Color");
	//show();

	propLabel = new QLabel(this);
	valueLabel = new QLabel(this);
	tagTextLabel = new QLabel(this);
	status = new QLabel(this);

	cBox = new QCheckBox("New line before Tag end.",this);

	propLabel->setText("Attributes");
	valueLabel->setText("Attribute Value (List or FreeText)");
	tagTextLabel->setText("Tag Text");
	status->setText("Waiting...");
	addColor->setText("Insert Color");


	close->setText("Done");
	add->setText("Add Attribute");
	//preview->setText("Preview");

	propLabel->setGeometry( QRect( 5, 10, 190, 20 ) );
	propertyKeysLB->setGeometry( QRect( 5, 30, 190, 20 ) );
	valueLabel->setGeometry( QRect( 5, 50, 190, 20 ) );
	possibleValuesLB->setGeometry( QRect( 5, 70, 190, 20 ) );
	lineEd->setGeometry( QRect( 5, 90,120,20 ) );
	addColor->setGeometry( QRect( 125, 90,70,20 ) );
	add->setGeometry( QRect( 5, 115, 190, 20 ) );

	tagTextLabel->setGeometry( QRect( 5, 135, 190, 20 ) );
	text->setGeometry( QRect( 5, 155, 190, 50 ) );

	cBox->setGeometry( QRect( 5, 210, 190, 20 ) );

	
	//preview->setGeometry( QRect( 5, 230, 95, 20 ) );
	close->setGeometry( QRect( 5, 230, 190, 20 ) );
	//close->setGeometry( QRect( 5, 240, 95, 20 ) );

	status->setGeometry( QRect( 5, 250, 190, 20 ) );

	connect(close, SIGNAL(clicked()), this, SLOT(cancel()));
	connect(add, SIGNAL(clicked()), this, SLOT(addValKeyPair()));
	connect(cBox, SIGNAL(clicked()), this, SLOT(setNewLine()));
	connect(addColor, SIGNAL(clicked()), this, SLOT(colorDial()));
	//connect(preview, SIGNAL(clicked()),this, SLOT(previewTag()));
}


void HTMLPropertyDialog::colorDial()
{
	QColor *color;
	color = new QColor(ColorDialog::getColor(QColor(255, 255, 255),this, "colorD"));

	QString *colorS = new QString();
	char *myRED = (char *)malloc(2*sizeof(char));
	char *myGREEN = (char *)malloc(2*sizeof(char));
	char *myBLUE = (char *)malloc(2*sizeof(char));
	sprintf(myRED,"%02x",color->red());
	sprintf(myGREEN,"%02x",color->green());
	sprintf(myBLUE,"%02x",color->blue());
	colorS->append("#");
	colorS->append(myRED);
	colorS->append(myGREEN);
	colorS->append(myBLUE);
	lineEd->setText(colorS->latin1());

	free(myRED);
	free(myGREEN);
	free(myBLUE);
	delete colorS;	
	delete color;


}

HTMLTag * HTMLPropertyDialog::getFinishedTag()
{
	if(text->text() != "")
	{
		tag->setTagText(text->text());
	}
	return tag;
}


void HTMLPropertyDialog::previewTag()
{
	if(text->text() != "")
	{
		tag->setTagText(text->text());
	}
	const QString *text = tag->getText();
	const char *txt = text->latin1();
	QString *myText = new QString("w:");
	myText->append(txt);
	myText->append("::");
	QMessageBox::about( this,"Tag preview",myText->latin1());
	printf("%s",txt);

}

void HTMLPropertyDialog::setNewLine()
{
	bool set = cBox->isChecked();
	tag->setAddNewLine(set);
}


void HTMLPropertyDialog::addValKeyPair()
{

	QString myText = lineEd->text();
	QString *statusT = new QString("Added property:");
	statusT->append(propertyKeysLB->currentText());
	if(myText.isEmpty())
	{
		tag->addProperty(propertyKeysLB->currentText(), possibleValuesLB->currentText());
		statusT->append("=\"");
		statusT->append(possibleValuesLB->currentText());
		statusT->append("\"");
	}
	else
	{
		statusT->append("=\"");
		tag->addProperty(propertyKeysLB->currentText(), myText);
		statusT->append(myText);
		lineEd->setText("");
		statusT->append("\"");
	}
	status->setText(statusT->latin1());
}

void HTMLPropertyDialog::setMarkedText(const char *myTxt)
{	
	text->append(myTxt);
}

void HTMLPropertyDialog::setTag(HTMLTag *myTag)
{


	propertyKeysLB->clear();
	possibleValuesLB->clear();
	text->clear();

	//set the Tag to the given pointer
	tag = myTag;


	//Set the dialog caption apropriatly
	QString *tmpText = tag->getKey();

	tmpText->prepend("TagEditor:");
	setCaption(*tmpText);
	delete tmpText;

	//Do a couple of test inserts

	HTMLProperty *myprops = tag->getPossibleProperties();

	int propCount = tag->getPossiblePropertyCount();

	//printf("Comment:%s",tag->getTagComment());
	for(int j = 0; j < propCount; j++)
	{
		propertyKeysLB->insertItem((myprops+j)->getName(),propertyKeysLB->count());
	}

	connect(propertyKeysLB ,SIGNAL(activated (int)), this, SLOT(changeList(int))  );


	//int chosen = propertyKeysLB->currentItem();


	changeList(propertyKeysLB->currentItem());

}

void HTMLPropertyDialog::changeList(int chose)
{
	if(possibleValuesLB->count() > 0)
	{
		possibleValuesLB->clear();
	}
	if(tag->getPossiblePropertyCount() > 0)
	{
		HTMLProperty *myprops = tag->getPossibleProperties();
		int count = (myprops+chose)->getpropCount();

		//-------------------------------------------
		FILE *myFile = fopen("htmlatzaurus.out","ab");
		putc('c',myFile);putc('o',myFile);putc('u',myFile);putc('n',myFile);putc('t',myFile);putc(':',myFile);
		fprintf(myFile,"%d",count);
		putc('\n',myFile);
		fclose(myFile);
		//------------------------------------------

		//printf("The Comment was:%s\n",(myprops+chose)->getTagComment());
		int i = 0;
		for(; i < count;)
		{
			possibleValuesLB->insertItem(myprops[chose].getValue(i), possibleValuesLB->count());
			i++;
		}

	}
}

void HTMLPropertyDialog::insertListItem(QString *key, QString *value)
{
	propertyKeysLB->insertItem(*key,propertyKeysLB->count());
	possibleValuesLB->insertItem(*value, possibleValuesLB->count());

}


void HTMLPropertyDialog::insertListItem(const char *key, const char *value)
{
	propertyKeysLB->insertItem(key,propertyKeysLB->count());
	possibleValuesLB->insertItem(value, possibleValuesLB->count());

}


void HTMLPropertyDialog::makeVisible(bool yn)
{
	if(yn)
	{
		status->setText("Waiting...");
		show();
	}
	else
	{
		hide();
	}
}


void HTMLPropertyDialog::cancel()
{
	propertyKeysLB->clear();
	possibleValuesLB->clear();
	emit done();
	hide();
}

HTMLPropertyDialog::~HTMLPropertyDialog()
{

}
