/*
    HTML@Zaurus HTML Editor for Sharp SL5xxx Linux PDA
    Copyright (C) 2002  Scott Taylor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/


#include "include/htmltag.h"

HTMLTag::HTMLTag()
{
	HTMLTag("no text specified!", 0);
}

HTMLTag::HTMLTag(const char *texT, bool UPPER)
{
	HTMLTag(texT,1, UPPER);
}

HTMLTag::HTMLTag(const char *texT, int hasEnd, bool UPPER)
{
	
	tagKey = new QString(texT);
	if(!UPPER)
	{
		tagKey = new QString(tagKey->lower().latin1());	
	}
	else
	{
		tagKey = new QString(tagKey->upper().latin1());
	}

	hasTagEnd = hasEnd;
	tagCount = 0;
	//tagKeyChar = "";
	//texT;
	NEWLINE = false;
	//tagKey = new QString(texT);
	tagKeyChar = tagKey->latin1();
	tagText = new QString();
	tagEnd = new QString();
	myText = new QString();
	comment = new QString();
	QString *tm2 = new QString(texT);
	
	if(!UPPER)
	{
		tm2 = new QString(tm2->lower().latin1());	
	}
	else
	{
		tm2 = new QString(tm2->upper().latin1());
	}

	
	const QString *temp = new QString(tm2->latin1());
	
	delete tm2;

	tagText->append("<");
	tagText->append(*temp);
	tagText->append(" ");

	tagEnd->append("</");
	tagEnd->append(*temp);
	tagEnd->append(">");
	myProperties = new HTMLProperty[MAX_P];

	delete temp;
}

void HTMLTag::setTagComment(const char *text)
{
	comment = new QString(text);
}

const char * HTMLTag::getTagComment()
{
	return comment->latin1();
}



void HTMLTag::setTagText(const char *text)
{
	myText = new QString(text);
}

void HTMLTag::addPossibleProperty(HTMLProperty *newProp)
{
	myProperties[tagCount] = *newProp;
	tagCount++;
}

int HTMLTag::getPossiblePropertyCount()
{
	return tagCount;
}

HTMLProperty * HTMLTag::getPossibleProperties()
{
	return myProperties;
}
void HTMLTag::addProperty(const char *key, const char *value)
{
	QString *keyQ = new QString(key);
	QString *valueQ = new QString(value);

	addProperty(keyQ,valueQ);
	
	//delete keyQ;
	//delete valueQ;
}


void HTMLTag::clearProperties()
{
	*tagText = tagText->right(0);
	tagText->append("<");
	tagText->append(tagKeyChar);

}

void HTMLTag::addProperty(QString *key, QString *value)
{
	QString *keyQ = key;
	const QString *valueQ = value;
	const QString *myEq = new QString("=\"");
	const QString *myEnd = new QString("\"");
	keyQ->prepend(" ");
	if(!valueQ->isEmpty())
	{
		keyQ->append(*myEq);
		keyQ->append(*valueQ);
		keyQ->append(*myEnd);
	}
	tagText->append(*keyQ);
	delete key;
	delete value;
	//delete valueQ;
	//delete keyQ;
}

void HTMLTag::setText(QString *text)
{

}

QString * HTMLTag::getText()
{
	const QString tmp = tagText->left(tagText->length());
	QString *tempText = new QString(tmp);

	tempText->append(">");
	if(NEWLINE)
	{
		tempText->append("\n");
	}
	if(hasTagEnd)
	{
		//printf("adding tag end %s\n",myText->latin1());
		if(!myText->isEmpty())
		{
			tempText->append(myText->latin1());
			if(NEWLINE)
			{
				tempText->append("\n");
			}
		}
		tempText->append(tagEnd->latin1());//left(tagEnd->length()));
	}
	tempText->append("\n");
	return tempText;


}
QString * HTMLTag::getKey()
{
	return new QString(tagKeyChar);
}

void HTMLTag::setAddNewLine(bool set)
{
	NEWLINE = set;
}


HTMLTag::~HTMLTag()
{
	//delete
}
