/*
    HTML@Zaurus HTML Editor for Sharp SL5xxx Linux PDA
    Copyright (C) 2002  Scott Taylor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/


#ifndef HTMLEDITOR_H
#define HTMLEDITOR_H
#include <qvariant.h>
#include <qwidget.h>
#include <qmainwindow.h>
#include <qmultilinedit.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qstring.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qstatusbar.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qcanvas.h>
#include <qcombobox.h>
#include <qxml.h>


/******************

My includes

******************/
#include "htmltag.h"
#include "htmlpropertydialog.h"
#include "htmlxml.h"
#include "filedialogs.h"
#include "htmlptagdialog.h"
#include "global.h"
#include "htmlconfig.h"
#include "htmlmlineedit.h"



class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QButtonGroup;
class QMultiLineEdit;
class QMainWindow;
class QToolBar;
class QToolButton;
class QPopupMenu;
class QMenuBar;
class QString;
class QFile;
class QTextStream;
class QStatusBar;
class QMessageBox;
class QCanvas;
class QCanvasText;
class QComboBox;

class HTMLEditor : public QMainWindow
{
	Q_OBJECT
	int whatIsShowing;

	public slots:
		void goodBye();
		void insertTag();
		void load(const char *filename);
		void load();
		void openPropertiesDialog();
		void newTag(const char *tagName);
		void showFileChooserSave();
		void showFileChooserLoad();
		void save();
		void saveAs();
		void newDocument();
		void blankDocument();
		void about();
		void colorDial();
		void preview();
		void insertMyTag();
		void insertPTag();
		void showConfig();
		void copy();
		void paste();
		void cut();
		void undo();
		void redo();


	public:
		//Constructors
		HTMLEditor( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );

		//Destructors
		~HTMLEditor();
		void docDone();
		void callSystem(char *string);

		//Public Functions
	protected:
		void closeEvent ( QCloseEvent * e );



	private:
		//Make the objects private (Nobody else needs to see them thats
		//why we have accessor methods....)
		QPushButton		*inserter;
		QToolBar		*fileTools;
		HTMLMLineEdit		*editor;
		QComboBox		*listBox;
		QString			*fileName;
  		HTMLTag			*tag;
		HTMLPropertyDialog 	*dialog;
		FileDialogS 		*f;
		//HTMLConfig			*confDial;
		int			tagCount;
		int			wrapAt;
		bool			CLOSE;
		bool			LOADING;
		bool			UPPER;
		//myAboutForm 		*aboutD;
		
		QString			*browser;
		QPopupMenu 		*tags;

		HTML_XMLParser *handler;
		HTMLPTagDialog *tagDialog;
		
		void tagInit();
		
		
		void save(const char *fileN);
		void loadConfig();
		
	signals:
		//void selected(int in);
};


#endif // HTMLEDITOR_H
